@@ -1,7 +1,7 @@
 # Copyright (c) Yuta Saito, Yusuke Narita, and ZOZO Technologies, Inc. All rights reserved.
 # Licensed under the Apache 2.0 License.
 
-"""Regression Model Class for Model-dependent OPE estimators."""
+"""Regression Model Class for Estimating Mean Reward Functions."""
 from dataclasses import dataclass
 from typing import Optional
 
@@ -14,30 +14,30 @@ from ..utils import check_bandit_feedback_inputs
 
 @dataclass
 class RegressionModel(BaseEstimator):
-    """Machine Learning model to estimate the mean reward function (:math:`\\mu(x, a) = \\mathbb{E} [Y(a) | X=x]`).
+    """Machine learning model to estimate the mean reward function (:math:`q(x,a):= \\mathbb{E}[r|x,a]`).
 
     Note
     -------
-    Reward (or outcome) :math:`Y` must be either binary or continuous.
+    Reward (or outcome) :math:`r` must be either binary or continuous.
 
     Parameters
     ------------
     base_model: BaseEstimator
-        Model class to be used to estimate the mean reward function.
+        A machine learning model used to estimate the mean reward function.
 
     n_actions: int
         Number of actions.
 
-    len_list: int, default: 1
-        Length of a list of recommended actions in each impression.
+    len_list: int, default=1
+        Length of a list of actions recommended in each impression.
         When Open Bandit Dataset is used, 3 should be set.
 
     action_context: array-like, shape (n_actions, dim_action_context), default=None
-        Context vector characterizing each action.
-        If not given, then one-hot encoding of the action variable is automatically used as `action_context`.
+        Context vector characterizing each action, vector representation of each action.
+        If not given, then one-hot encoding of the action variable is automatically used.
 
     fitting_method: str, default='normal'
-        Method to fit the regression method.
+        Method to fit the regression model.
         Must be one of ['normal', 'iw', 'mrdr'] where 'iw' stands for importance weighting and
         'mrdr' stands for more robust doubly robust.
 
@@ -66,7 +66,7 @@ class RegressionModel(BaseEstimator):
             "normal",
             "iw",
             "mrdr",
-        ], f"fitting method must be one of 'normal', 'iw', or 'mrdr', but {self.fitting_method} is given"
+        ], f"fitting_method must be one of 'normal', 'iw', or 'mrdr', but {self.fitting_method} is given"
         assert self.n_actions > 1 and isinstance(
             self.n_actions, int
         ), f"n_actions must be an integer larger than 1, but {self.n_actions} is given"
@@ -93,25 +93,26 @@ class RegressionModel(BaseEstimator):
         Parameters
         ----------
         context: array-like, shape (n_rounds, dim_context)
-            Context vectors in the given training logged bandit feedback.
+            Context vectors in each round, i.e., :math:`x_t`.
 
         action: array-like, shape (n_rounds,)
-            Selected actions by behavior policy in the given training logged bandit feedback.
+            Action sampled by a behavior policy in each round of the logged bandit feedback, i.e., :math:`a_t`.
 
         reward: array-like, shape (n_rounds,)
-            Observed rewards in the given training logged bandit feedback.
+            Observed rewards (or outcome) in each round, i.e., :math:`r_t`.
 
-        pscore: Optional[np.ndarray], default: None
-            Propensity scores, the probability of selecting each action by behavior policy,
-            in the given training logged bandit feedback.
+        pscore: array-like, shape (n_rounds,), default=None
+            Action choice probabilities (propensity score) of a behavior policy
+            in the training logged bandit feedback.
+            When None is given, the the behavior policy is assumed to be a uniform one.
 
         position: array-like, shape (n_rounds,), default=None
-            Positions of each round in the given training logged bandit feedback.
-            If None is given, a learner assumes that there is only one position.
-            When `len_list` > 1, position has to be set.
+            Positions of each round in the given logged bandit feedback.
+            If None is set, a regression model assumes that there is only one position.
+            When `len_list` > 1, this position argument has to be set.
 
-        action_dist: array-like shape (n_rounds, n_actions, len_list), default=None
-            Distribution over actions, i.e., probability of items being selected at each position by the evaluation policy (can be deterministic).
+        action_dist: array-like, shape (n_rounds, n_actions, len_list), default=None
+            Action choice probabilities by the evaluation policy (can be deterministic), i.e., :math:`\\pi_e(a_t|x_t)`.
             When either of 'iw' or 'mrdr' is used as the 'fitting_method' argument, then `action_dist` must be given.
 
         """
@@ -123,20 +124,26 @@ class RegressionModel(BaseEstimator):
             position=position,
             action_context=self.action_context,
         )
+        n_rounds = context.shape[0]
+
         if self.len_list == 1:
             position = np.zeros_like(action)
         else:
             assert (
-                position is not None
-            ), "position has to be set when len_list is larger than 1"
+                isinstance(position, np.ndarray) and position.ndim == 1
+            ), f"when len_list > 1, position must be a 1-dimensional ndarray"
         if self.fitting_method in ["iw", "mrdr"]:
             assert (
-                action_dist is not None
-            ), "When either 'iw' or 'mrdr' is used as the 'fitting_method' argument, then action_dist must be given"
-            assert (
-                pscore is not None
-            ), "When either 'iw' or 'mrdr' is used as the 'fitting_method' argument, then pscore must be given"
-        n_data = context.shape[0]
+                isinstance(action_dist, np.ndarray) and action_dist.ndim == 3
+            ), f"when fitting_method is either 'iw' or 'mrdr', action_dist must be a 3-dimensional ndarray"
+            assert action_dist.shape == (
+                n_rounds,
+                self.n_actions,
+                self.len_list,
+            ), f"shape of action_dist must be (n_rounds, n_actions, len_list)=({n_rounds, self.n_actions, self.len_list})"
+            if pscore is None:
+                pscore = np.ones_like(action) / self.n_actions
+
         for position_ in np.arange(self.len_list):
             idx = position == position_
             X = self._pre_process_for_reg_model(
@@ -149,7 +156,9 @@ class RegressionModel(BaseEstimator):
                 self.base_model_list[position_].fit(X, reward[idx])
             else:
                 action_dist_at_position = action_dist[
-                    np.arange(n_data), action, position_ * np.ones(n_data, dtype=int)
+                    np.arange(n_rounds),
+                    action,
+                    position_ * np.ones(n_rounds, dtype=int),
                 ][idx]
                 if self.fitting_method == "iw":
                     sample_weight = action_dist_at_position / pscore[idx]
@@ -157,11 +166,9 @@ class RegressionModel(BaseEstimator):
                         X, reward[idx], sample_weight=sample_weight
                     )
                 elif self.fitting_method == "mrdr":
-                    sample_weight = (
-                        action_dist_at_position
-                        * (1.0 - pscore[idx])
-                        / (pscore[idx] ** 2)
-                    )
+                    sample_weight = action_dist_at_position
+                    sample_weight *= 1.0 - pscore[idx]
+                    sample_weight /= pscore[idx] ** 2
                     self.base_model_list[position_].fit(
                         X, reward[idx], sample_weight=sample_weight
                     )
@@ -177,7 +184,7 @@ class RegressionModel(BaseEstimator):
         Returns
         -----------
         estimated_rewards_by_reg_model: array-like, shape (n_rounds_of_new_data, n_actions, len_list)
-            Estimated expected rewards for new data given each item and position by the regression model.
+            Estimated expected rewards for new data by the regression model.
 
         """
         n_rounds_of_new_data = context.shape[0]
@@ -213,48 +220,64 @@ class RegressionModel(BaseEstimator):
         position: Optional[np.ndarray] = None,
         action_dist: Optional[np.ndarray] = None,
         n_folds: int = 1,
+        random_state: Optional[int] = None,
     ) -> None:
-        """Fit the regression model on given logged bandit feedback data and then predict the mean reward function of the same data.
+        """Fit the regression model on given logged bandit feedback data and predict the reward function of the same data.
 
         Note
         ------
         When `n_folds` is larger than 1, then the cross-fitting procedure is applied.
-        See the reference for the details of the cross-fitting.
+        See the reference for the details about the cross-fitting technique.
 
         Parameters
         ----------
         context: array-like, shape (n_rounds, dim_context)
-            Context vectors in the given training logged bandit feedback.
+            Context vectors in each round, i.e., :math:`x_t`.
 
         action: array-like, shape (n_rounds,)
-            Selected actions by behavior policy in the given training logged bandit feedback.
+            Action sampled by a behavior policy in each round of the logged bandit feedback, i.e., :math:`a_t`.
 
         reward: array-like, shape (n_rounds,)
-            Observed rewards in the given training logged bandit feedback.
+            Observed rewards (or outcome) in each round, i.e., :math:`r_t`.
 
-        pscore: Optional[np.ndarray], default: None
-            Propensity scores, the probability of selecting each action by behavior policy,
-            in the given training logged bandit feedback.
+        pscore: array-like, shape (n_rounds,), default=None
+            Action choice probabilities (propensity score) of a behavior policy
+            in the training logged bandit feedback.
+            When None is given, the the behavior policy is assumed to be a uniform one.
 
         position: array-like, shape (n_rounds,), default=None
-            Positions of each round in the given training logged bandit feedback.
-            If None is given, a learner assumes that there is only one position.
-            When `len_list` > 1, position has to be set.
+            Positions of each round in the given logged bandit feedback.
+            If None is set, a regression model assumes that there is only one position.
+            When `len_list` > 1, this position argument has to be set.
 
-        action_dist: array-like shape (n_rounds, n_actions, len_list), default=None
-            Distribution over actions, i.e., probability of items being selected at each position by the evaluation policy (can be deterministic).
+        action_dist: array-like, shape (n_rounds, n_actions, len_list), default=None
+            Action choice probabilities by the evaluation policy (can be deterministic), i.e., :math:`\\pi_e(a_t|x_t)`.
             When either of 'iw' or 'mrdr' is used as the 'fitting_method' argument, then `action_dist` must be given.
 
         n_folds: int, default=1
             Number of folds in the cross-fitting procedure.
-            When 1 is given, then the regression model is trained on the whole logged bandit feedback data.
+            When 1 is given, the regression model is trained on the whole logged bandit feedback data.
+
+        random_state: int, default=None
+            `random_state` affects the ordering of the indices, which controls the randomness of each fold.
+            See https://scikit-learn.org/stable/modules/generated/sklearn.model_selection.KFold.html for the details.
 
         Returns
         -----------
         estimated_rewards_by_reg_model: array-like, shape (n_rounds, n_actions, len_list)
-            Estimated expected rewards for the given logged bandit feedback at each item and position by the regression model.
+            Estimated expected rewards for new data by the regression model.
 
         """
+        check_bandit_feedback_inputs(
+            context=context,
+            action=action,
+            reward=reward,
+            pscore=pscore,
+            position=position,
+            action_context=self.action_context,
+        )
+        n_rounds = context.shape[0]
+
         assert n_folds > 0 and isinstance(
             n_folds, int
         ), f"n_folds must be a positive integer, but {n_folds} is given"
@@ -262,15 +285,19 @@ class RegressionModel(BaseEstimator):
             position = np.zeros_like(action)
         else:
             assert (
-                position is not None
-            ), "position has to be set when len_list is larger than 1"
+                isinstance(position, np.ndarray) and position.ndim == 1
+            ), f"when len_list > 1, position must be a 1-dimensional ndarray"
         if self.fitting_method in ["iw", "mrdr"]:
             assert (
-                action_dist is not None
-            ), "When either 'iw' or 'mrdr' is used as the 'fitting_method' argument, then action_dist must be given"
-            assert (
-                pscore is not None
-            ), "When either 'iw' or 'mrdr' is used as the 'fitting_method' argument, then pscore must be given"
+                isinstance(action_dist, np.ndarray) and action_dist.ndim == 3
+            ), f"when fitting_method is either 'iw' or 'mrdr', action_dist must be a 3-dimensional ndarray"
+            assert action_dist.shape == (
+                n_rounds,
+                self.n_actions,
+                self.len_list,
+            ), f"shape of action_dist must be (n_rounds, n_actions, len_list)={n_rounds, self.n_actions, self.len_list}, but is {action_dist.shape}"
+        if pscore is None:
+            pscore = np.ones_like(action) / self.n_actions
 
         if n_folds == 1:
             self.fit(
@@ -284,11 +311,11 @@ class RegressionModel(BaseEstimator):
             return self.predict(context=context)
         else:
             estimated_rewards_by_reg_model = np.zeros(
-                (context.shape[0], self.n_actions, self.len_list)
+                (n_rounds, self.n_actions, self.len_list)
             )
-        skf = KFold(n_splits=n_folds, shuffle=True)
-        skf.get_n_splits(context)
-        for train_idx, test_idx in skf.split(context, reward):
+        kf = KFold(n_splits=n_folds, shuffle=True, random_state=random_state)
+        kf.get_n_splits(context)
+        for train_idx, test_idx in kf.split(context):
             action_dist_tr = (
                 action_dist[train_idx] if action_dist is not None else action_dist
             )
@@ -318,13 +345,13 @@ class RegressionModel(BaseEstimator):
         Parameters
         -----------
         context: array-like, shape (n_rounds,)
-            Context vectors in the given training logged bandit feedback.
+            Context vectors in the training logged bandit feedback.
 
         action: array-like, shape (n_rounds,)
-            Selected actions by behavior policy in the given training logged bandit feedback.
+            Action sampled by a behavior policy in each round of the logged bandit feedback, i.e., :math:`a_t`.
 
         action_context: array-like, shape shape (n_actions, dim_action_context)
-            Context vector characterizing each action.
+            Context vector characterizing each action, vector representation of each action.
 
         """
         return np.c_[context, action_context[action]]
